"""Tests for the management commands of the ``object_events`` app."""
from django.contrib.auth.models import SiteProfileNotAvailable
from django.core.management import call_command
from django.test import TestCase
from django.test.utils import override_settings

from mailer.models import Message
from nose.tools import raises

from .factories import ObjectEventFactory, TestProfileFactory
from ..models import UserAggregation


class SendEventEmailsTestCase(TestCase):
    """Tests for the ``send_event_emails`` management command."""
    longMessage = True

    @raises(SystemExit)
    def test_wrong_aggregation_class(self):
        with self.settings(
            OBJECT_EVENTS_USER_AGGREGATION_CLASS=('test_app.models.'
                                                  'EmptyAggregation')):
            call_command('send_event_emails', 'realtime')

    @raises(SystemExit)
    def test_missing_argument(self):
        self.assertFalse(call_command('send_event_emails'))

    @raises(SiteProfileNotAvailable)
    @override_settings(AUTH_PROFILE_MODULE=False)
    def test_missing_user_profile(self):
        UserAggregation()

    @raises(SiteProfileNotAvailable)
    @override_settings(AUTH_PROFILE_MODULE='test')
    def test_wrong_user_profile_definition(self):
        UserAggregation()

    @raises(SiteProfileNotAvailable)
    @override_settings(AUTH_PROFILE_MODULE='test.Test')
    def test_wrong_user_profile_app(self):
        UserAggregation()

    @raises(SiteProfileNotAvailable)
    @override_settings(AUTH_PROFILE_MODULE='test_app.NonExistingProfile')
    def test_wrong_user_profile_model(self):
        UserAggregation()

    @raises(SiteProfileNotAvailable)
    @override_settings(AUTH_PROFILE_MODULE='test_app.WrongTestProfile')
    def test_user_profile_model_without_interval_field(self):
        UserAggregation()

    def test_realtime(self):
        self.assertFalse(call_command('send_event_emails', 'realtime'))
        profile = TestProfileFactory(interval='realtime')
        self.assertFalse(call_command('send_event_emails', 'realtime'))
        first_object_event = ObjectEventFactory(user=profile.user)
        self.assertFalse(call_command('send_event_emails', 'realtime'))
        self.assertEqual(Message.objects.all().count(), 1)
        ObjectEventFactory(user=profile.user)
        ObjectEventFactory(user=profile.user,
                           event_type=first_object_event.event_type)
        other_profile = TestProfileFactory(interval='realtime')
        ObjectEventFactory(user=other_profile.user,
                           event_type=first_object_event.event_type)
        ObjectEventFactory(user=other_profile.user,
                           event_type=first_object_event.event_type)
        self.assertFalse(call_command('send_event_emails', 'realtime'))
        self.assertEqual(Message.objects.all().count(), 3)

    def test_daily(self):
        self.assertFalse(call_command('send_event_emails', 'daily'))
        profile = TestProfileFactory(interval='daily')
        self.assertFalse(call_command('send_event_emails', 'daily'))
        ObjectEventFactory(user=profile.user)
        self.assertFalse(call_command('send_event_emails', 'daily'))
        self.assertEqual(Message.objects.all().count(), 1)

    def test_weekly(self):
        self.assertFalse(call_command('send_event_emails', 'weekly'))
        profile = TestProfileFactory(interval='weekly')
        self.assertFalse(call_command('send_event_emails', 'weekly'))
        ObjectEventFactory(user=profile.user)
        self.assertFalse(call_command('send_event_emails', 'weekly'))
        self.assertEqual(Message.objects.all().count(), 1)

    def test_monthly(self):
        self.assertFalse(call_command('send_event_emails', 'monthly'))
        profile = TestProfileFactory(interval='monthly')
        self.assertFalse(call_command('send_event_emails', 'monthly'))
        ObjectEventFactory(user=profile.user)
        self.assertFalse(call_command('send_event_emails', 'monthly'))
        self.assertEqual(Message.objects.all().count(), 1)



"""Tests for tags of the ``object_events``` application."""
from django.contrib.auth.models import AnonymousUser
from django.contrib.sessions.middleware import SessionMiddleware
from django.template.context import RequestContext
from django.test import TestCase
from django.test.client import RequestFactory

from django_libs.tests.factories import UserFactory

from ..templatetags.object_events_tags import render_notifications
from .factories import ObjectEventFactory


class RenderNotificationsTestCase(TestCase):
    """Tests for the ``render_notifications`` tag."""
    longMessage = True

    def test_tag(self):
        # create context mock
        request = RequestFactory().get('/')
        request.user = AnonymousUser()
        SessionMiddleware().process_request(request)
        request.session.save()
        context = RequestContext(request)

        # Returns empty dict if there is no authenticated user
        self.assertEqual(render_notifications(context), {})

        # If authenticated return an info
        request.user = UserFactory()
        self.assertEqual(render_notifications(context),
                         {'authenticated': True})

        # Returns notifications.html
        event = ObjectEventFactory(user=request.user)
        self.assertEqual(render_notifications(context).get('notifications'),
                         [event])
        self.assertEqual(render_notifications(context).get('unread_amount'), 1)
        self.assertEqual(render_notifications(context).get('authenticated'),
                         True)



"""Admin classes for the ``object_events`` app."""
from django.contrib import admin

from .models import ObjectEvent, ObjectEventType


class ObjectEventTypeAdmin(admin.ModelAdmin):
    list_display = ['title', ]


class ObjectEventAdmin(admin.ModelAdmin):
    list_display = [
        'user_email', 'content_object', 'type_title', 'creation_date', ]

    def content_object(self, obj):
        return obj.content_object
    content_object.short_description = 'Content object'

    def user_email(self, obj):
        return obj.user.email
    user_email.short_description = 'User'

    def type_title(self, obj):
        return obj.event_type.title
    type_title.short_description = 'Type'


admin.site.register(ObjectEvent, ObjectEventAdmin)
admin.site.register(ObjectEventType, ObjectEventTypeAdmin)



"""Settings of the ``object_events``` application."""
from django.conf import settings


def get_user_aggregation_class():
    """This needs to be a lazy setting because of our tests."""
    return getattr(
        settings,
        'OBJECT_EVENTS_USER_AGGREGATION_CLASS',
        'object_events.models.UserAggregation',
    )

USER_AGGREGATION_CLASS = lambda: get_user_aggregation_class()
PAGINATION_ITEMS = getattr(settings, 'OBJECT_EVENTS_PAGINATION_ITEMS', 30)



# flake8: noqa
# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'ObjectEvent.event_content_type'
        db.add_column('object_events_objectevent', 'event_content_type',
                      self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='event_objects', null=True, to=orm['contenttypes.ContentType']),
                      keep_default=False)

        # Adding field 'ObjectEvent.event_object_id'
        db.add_column('object_events_objectevent', 'event_object_id',
                      self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True),
                      keep_default=False)


        # Changing field 'ObjectEvent.object_id'
        db.alter_column('object_events_objectevent', 'object_id', self.gf('django.db.models.fields.PositiveIntegerField')(null=True))

        # Changing field 'ObjectEvent.user'
        db.alter_column('object_events_objectevent', 'user_id', self.gf('django.db.models.fields.related.ForeignKey')(null=True, to=orm['auth.User']))

        # Changing field 'ObjectEvent.content_type'
        db.alter_column('object_events_objectevent', 'content_type_id', self.gf('django.db.models.fields.related.ForeignKey')(null=True, to=orm['contenttypes.ContentType']))

    def backwards(self, orm):
        # Deleting field 'ObjectEvent.event_content_type'
        db.delete_column('object_events_objectevent', 'event_content_type_id')

        # Deleting field 'ObjectEvent.event_object_id'
        db.delete_column('object_events_objectevent', 'event_object_id')


        # User chose to not deal with backwards NULL issues for 'ObjectEvent.object_id'
        raise RuntimeError("Cannot reverse this migration. 'ObjectEvent.object_id' and its values cannot be restored.")

        # User chose to not deal with backwards NULL issues for 'ObjectEvent.user'
        raise RuntimeError("Cannot reverse this migration. 'ObjectEvent.user' and its values cannot be restored.")

        # User chose to not deal with backwards NULL issues for 'ObjectEvent.content_type'
        raise RuntimeError("Cannot reverse this migration. 'ObjectEvent.content_type' and its values cannot be restored.")

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'object_events.objectevent': {
            'Meta': {'object_name': 'ObjectEvent'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'event_content_objects'", 'null': 'True', 'to': "orm['contenttypes.ContentType']"}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'event_content_type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'event_objects'", 'null': 'True', 'to': "orm['contenttypes.ContentType']"}),
            'event_object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'events'", 'to': "orm['object_events.ObjectEventType']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'events'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'object_events.objecteventtype': {
            'Meta': {'object_name': 'ObjectEventType'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '256'})
        }
    }

    complete_apps = ['object_events']



"""Urls for the ``object_events`` app."""
from django.conf.urls.defaults import patterns, url

from .views import ObjectEventsMarkView, ObjectEventsListView


urlpatterns = patterns(
    '',
    url(r'^mark/$', ObjectEventsMarkView.as_view(), name='object_events_mark'),
    url(r'^$', ObjectEventsListView.as_view(), name='object_events_list'),
)



"""Views for the ``object_events`` app."""
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.http import Http404, HttpResponse
from django.utils.decorators import method_decorator
from django.views.generic import ListView, RedirectView

from .models import ObjectEvent
from .app_settings import PAGINATION_ITEMS


def is_integer(mark_string):
    """Function to check if a supposed pk is an integer."""
    try:
        mark_id = int(mark_string)
    except ValueError:
        return False
    return mark_id


class ObjectEventsListView(ListView):
    """View to display a defined amount of notifications."""
    paginate_by = PAGINATION_ITEMS

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        self.user = request.user
        return super(ObjectEventsListView, self).dispatch(request, *args,
                                                          **kwargs)

    def get_queryset(self):
        return ObjectEvent.objects.filter(user=self.user)


class ObjectEventsMarkView(RedirectView):
    """View to mark a set of object events as read."""
    permanent = False

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        if not request.method == 'POST':
            raise Http404
        if request.POST.get('single_mark'):
            mark_id = is_integer(request.POST.get('single_mark'))
            if not mark_id:
                raise Http404
            try:
                event = ObjectEvent.objects.get(user=request.user, pk=mark_id)
            except ObjectEvent.DoesNotExist:
                raise Http404
            event.read_by_user = True
            event.save()
            if request.is_ajax():
                return HttpResponse('marked')
        elif request.POST.get('bulk_mark'):
            event_pks = request.POST.get('bulk_mark').split(',')
            event_pks = [n for n in event_pks if is_integer(n)]
            if event_pks:
                events = ObjectEvent.objects.filter(
                    pk__in=event_pks, user=request.user)
                events.update(read_by_user=True)
                if request.is_ajax():
                    return HttpResponse('marked')
        return super(ObjectEventsMarkView, self).dispatch(request, *args,
                                                          **kwargs)

    def get_redirect_url(self, **kwargs):
        if self.request.POST.get('next'):
            return self.request.POST.get('next')
        return reverse('object_events_list')



"""
Custom admin command to send fresh object_events via email.

The interval is provided in an argument.
Options:
  realtime -> e.g. minute-by-minute
  daily    -> e.g. every day at midnight
  weekly   -> e.g. every sunday at 3 p.m.
  monthly  -> e.g. every last sunday of a month at 5 p.m.

The command always iterates over all events that have email_sent=False, but if
your app allows users to change their notification interval in their
UserProfile providing this parameter will call a different method on your
UserAggregation class implementation and therefore will include only a subset
of events (only for certain users).

"""
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.utils import timezone

from django_libs.loaders import load_member_from_setting
from django_libs.utils_email import send_email

from ...models import ObjectEvent, UserAggregationBase
from ... import app_settings


class Command(BaseCommand):
    """Class for the send_event_emails admin command."""
    def send_mail_to_user(self, email_context, to):
        """
        Function to send the digest to the user.

        First we check the preffered email of the User instance. You can
        add this function in your defined User Profile. Therefore check
        Django's AUTH_PROFILE_MODULE setting. If you haven't defined this
        function or defined a Profile module the email of the User instance is
        used.

        """
        email = to.email
        if (hasattr(to, 'get_profile')
                and hasattr(to.get_profile(), 'get_preferred_email')):
            email = to.get_profile().get_preferred_email()
        if email:
            send_email(
                '',
                {'event_types': email_context},
                'object_events/email/subject.html',
                'object_events/email/body.html',
                settings.FROM_EMAIL,
                [email],
            )
            self.sent_emails += 1

    def handle(self, interval='', **options):
        """Handles the send_event_emails admin command."""
        # Check if there is an aggregation class defined.
        start_of_command = timezone.now()
        if not interval in ('realtime', 'daily', 'weekly', 'monthly'):
            raise CommandError('Please provide a valid interval argument'
                               ' (realtime, daily, weekly, monthly)')
        aggregation = load_member_from_setting(
            'USER_AGGREGATION_CLASS', app_settings)()
        if not isinstance(aggregation, UserAggregationBase):
            raise CommandError(
                'Your user aggregation class must inherit UserAggregationBase')
        # Check interval argument and functions in the aggregation class.
        try:
            users = getattr(aggregation, 'get_users')(interval)
        except AttributeError:
            raise CommandError('Function get_users() not defined.')
        if not users:
            print('No users to send a {0} email.'.format(interval))
            return
        # Get all events , which hasn't been sent yet.
        object_events = ObjectEvent.objects.filter(
            email_sent=False, user__pk__in=users).order_by('user__pk')
        if not object_events:
            print('No events to send.')
            return
        email_context = {}
        current_user = None
        self.sent_emails = 0
        for object_event in object_events:
            if current_user != object_event.user and email_context:
                self.send_mail_to_user(email_context, current_user)
                email_context = {}
            current_user = object_event.user
            if email_context.get(object_event.event_type.title):
                email_context[object_event.event_type.title].append(
                    object_event)
            else:
                email_context.update({'{0}'.format(
                    object_event.event_type.title): [object_event]})
            object_event.email_sent = True
            object_event.save()
        # Send email even for the last user in the queryset, who cannot have
        # a follower
        if email_context:
            self.send_mail_to_user(email_context, current_user)
        print('The command took {0} seconds to finish. Sent {1} emails for {2}'
              ' events.'.format((timezone.now() - start_of_command).seconds,
                                self.sent_emails, object_events.count()))



"""Models for the ``object_events`` app."""
from django.conf import settings
from django.contrib.auth.models import SiteProfileNotAvailable
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.template.defaultfilters import date
from django.utils.timesince import timesince
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _


"""
Interval the user will be notified.

  realtime -> e.g. minute-by-minute
  daily    -> e.g. every day at midnight
  weekly   -> e.g. every sunday at 3 p.m.
  monthly  -> e.g. every last sunday of a month at 5 p.m.

For each interval that you allow in your app make sure that you define a
cronjob that is scheduled at this interval and called with the interval name
as it's parameter.

"""
NOTIFICATION_INTERVALS = (
    ('realtime', _('realtime')),
    ('daily', _('daily')),
    ('weekly', _('weekly')),
    ('monthly', _('monthly')),
)


class UserAggregationBase(object):
    """Base aggregation class to inherit from."""
    def get_users(self, interval):
        """Function to delegate user aggregation."""
        return getattr(self, 'get_{0}_users'.format(interval))()

    def get_realtime_users(self):
        """Function to aggregate users, which will be notified in realtime."""
        raise NotImplementedError()

    def get_daily_users(self):
        """Function to aggregate users, which will be notified daily."""
        raise NotImplementedError()

    def get_weekly_users(self):
        """Function to aggregate users, which will be notified weekly."""
        raise NotImplementedError()

    def get_monthly_users(self):
        """Function to aggregate users, which will be notified monthly."""
        raise NotImplementedError()


class UserAggregation(UserAggregationBase):
    """Class to aggregate 'realtime', 'daily', 'weekly', 'monthly' users."""
    def __init__(self):
        """Checks if there's a user profile."""
        if not getattr(settings, 'AUTH_PROFILE_MODULE', False):
            raise SiteProfileNotAvailable(
                'You need to set AUTH_PROFILE_MODULE in your project settings')
        try:
            app_label, model_name = settings.AUTH_PROFILE_MODULE.split('.')
        except ValueError:
            raise SiteProfileNotAvailable(
                'app_label and model_name should be separated by a dot in'
                ' the AUTH_PROFILE_MODULE setting')
        self.model = models.get_model(app_label, model_name)
        if self.model is None:
            raise SiteProfileNotAvailable(
                'Unable to load the profile model, check'
                ' AUTH_PROFILE_MODULE in your project settings')
        if not hasattr(self.model(), 'interval'):
            raise SiteProfileNotAvailable('Model has no field "interval"')

    def get_realtime_users(self):
        """Function to aggregate users, which will be notified in realtime."""
        return self.model.objects.filter(interval='realtime').values_list(
            'user__pk', flat=True)

    def get_daily_users(self):
        """Function to aggregate users, which will be notified daily."""
        return self.model.objects.filter(interval='daily').values_list(
            'user__pk', flat=True)

    def get_weekly_users(self):
        """Function to aggregate users, which will be notified weekly."""
        return self.model.objects.filter(interval='weekly').values_list(
            'user__pk', flat=True)

    def get_monthly_users(self):
        """Function to aggregate users, which will be notified monthly."""
        return self.model.objects.filter(interval='monthly').values_list(
            'user__pk', flat=True)


class ObjectEventType(models.Model):
    """
    Masterdata table containing event types.

    :title: Unique title of this event type. This will be used to decide which
      notification message to display or which partial to render.

    """
    title = models.SlugField(
        max_length=256,
        unique=True,
        verbose_name=_('Title'),
        help_text=_('Please use a slugified name, e.g. "student-news".'),
    )

    class Meta:
        ordering = ['title']

    def __unicode__(self):
        return u'{0}'.format(self.title)


class ObjectEvent(models.Model):
    """
    An event created by a user related to any object.

    :user: FK to the user who created this event. Leave this empty if this
      event was created by no user but automatically.
    :creation_date: Creation date of this event.
    :event_type: Type of this event.
    :email_sent: True, if user has received this event via email.
    :read_by_user: True, if user has noticed this event.
    :content_object: Generic foreign key to the object this event is attached
      to. Leave this empty if it is a global event.
    :event_content_object: Generic foreign key to the object that has been
      created by this event. Leave this empty if the event did not create any
      object.
    :additional_text: Plain text, which can be added to the notification,
      object, e.g. if you want to inform somebody that something has been
      deleted.

    """
    user = models.ForeignKey(
        'auth.User',
        verbose_name=_('User'),
        related_name='object_events',
        null=True, blank=True,
    )

    creation_date = models.DateTimeField(
        auto_now_add=True,
        verbose_name=_('Creation date'),
    )

    event_type = models.ForeignKey(
        ObjectEventType,
        verbose_name=_('Type'),
        related_name='events',
    )

    email_sent = models.BooleanField(
        verbose_name=_('Has been sent in an email?'),
        default=False,
    )

    read_by_user = models.BooleanField(
        verbose_name=_('Has been noticed by the user?'),
        default=False,
    )

    # Generic FK to the object this event is attached to
    content_type = models.ForeignKey(
        ContentType,
        related_name='event_content_objects',
        null=True, blank=True
    )
    object_id = models.PositiveIntegerField(null=True, blank=True)
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    # Generic FK to the object that got created by this event
    event_content_type = models.ForeignKey(
        ContentType,
        related_name='event_objects',
        null=True, blank=True
    )
    event_object_id = models.PositiveIntegerField(null=True, blank=True)
    event_content_object = generic.GenericForeignKey(
        'event_content_type', 'event_object_id')

    additional_text = models.CharField(
        max_length=128,
        verbose_name=_('Additional text'),
        blank=True,
    )

    class Meta:
        ordering = ['-creation_date']

    @staticmethod
    def create_event(user, content_object, event_content_object=None,
                     event_type='', additional_text=''):
        """
        Creates an event for the given user, object and type.

        If the type doesn't exist, yet, it will be created, so make sure that
        you don't have any typos in your type title.

        :param user: The user who created this event.
        :param content_object: The object this event is attached to.
        :param event_content_object: The object that was created by this event.
        :event_type: String representing the type of this event.
        :additional_text: Additional text.

        """
        event_type_obj, created = ObjectEventType.objects.get_or_create(
            title=event_type)
        kwargs = {
            'user': user,
            'content_object': content_object,
            'event_type': event_type_obj,
            'additional_text': additional_text,
        }
        if event_content_object is not None:
            kwargs.update({'event_content_object': event_content_object})
        obj = ObjectEvent.objects.create(**kwargs)
        return obj

    def __unicode__(self):
        return u'{0}'.format(self.content_object)

    def get_timesince(self):
        delta = (now() - self.creation_date)
        if delta.days <= 1:
            return '{0} ago'.format(timesince(self.creation_date, now()))
        if self.creation_date.year != now().year:
            return date(self.creation_date, 'd F Y')
        return date(self.creation_date, 'd F')



"""Tests for the models of the ``object_events`` app."""
from django.template.defaultfilters import date
from django.test import TestCase
from django.utils.timezone import now, timedelta

from django_libs.tests.factories import UserFactory
from nose.tools import raises

from ..models import ObjectEvent, ObjectEventType, UserAggregationBase
from .factories import ObjectEventFactory, ObjectEventTypeFactory


class ObjectEventTypeTestCase(TestCase):
    """Tests for the ``ObjectEventType`` model class."""
    def test_model(self):
        """Should be able to instantiate and save the model."""
        obj = ObjectEventTypeFactory()
        self.assertTrue(obj.pk)


class ObjectEventTestCase(TestCase):
    """Tests for the ``ObjectEvent`` model class."""
    def test_model(self):
        """Should be able to instantiate and save the model."""
        obj = ObjectEventFactory()
        self.assertTrue(obj.pk)

    def test_create_event(self):
        user = UserFactory()
        content_object = UserFactory()
        event = ObjectEvent.create_event(user, content_object)
        self.assertIsInstance(event, ObjectEvent)
        self.assertEqual(ObjectEvent.objects.all().count(), 1)
        self.assertEqual(ObjectEventType.objects.all().count(), 1)

        # Test with a new content_object, created for this event
        event_content_object = UserFactory()
        event = ObjectEvent.create_event(user, content_object,
                                         event_content_object)
        self.assertEqual(event.event_content_object, event_content_object)

    def test_get_timesince(self):
        # Just created object_event
        object_event = ObjectEventFactory()
        self.assertEqual(object_event.get_timesince(), '0 minutes ago')

        # 'Young' object_event
        object_event.creation_date = now() - timedelta(days=1)
        self.assertEqual(object_event.get_timesince(), '1 day ago')

        # 'Teen' object_event
        object_event.creation_date = now() - timedelta(days=5)
        self.assertEqual(object_event.get_timesince(), date(
            object_event.creation_date, 'd F'))

        # 'Old' object_event
        object_event.creation_date = now() - timedelta(days=365)
        self.assertEqual(object_event.get_timesince(), date(
            object_event.creation_date, 'd F Y'))


class UserAggregationBaseTestCase(TestCase):
    """Tests for the ``UserAggregationBase`` aggregation class."""
    @raises(NotImplementedError)
    def test_get_realtime_users(self):
        UserAggregationBase().get_users('realtime')

    @raises(NotImplementedError)
    def test_get_daily_users(self):
        UserAggregationBase().get_users('daily')

    @raises(NotImplementedError)
    def test_get_weekly_users(self):
        UserAggregationBase().get_users('weekly')

    @raises(NotImplementedError)
    def test_get_monthly_users(self):
        UserAggregationBase().get_users('monthly')



"""Dummy models to be used in test cases of the ``object_events`` app."""
from django.db import models

from object_events.models import NOTIFICATION_INTERVALS


class DummyModel(models.Model):
    """Dummy model to be used in test cases of the ``object_events`` app."""
    name = models.CharField(max_length=256, blank=True)


class TestProfile(models.Model):
    """
    Enhanced ``User`` model to save and test notification intervals.

    :user: Reference to Django's User model.
    :interval: Setting to define the notification interval.

    """
    user = models.ForeignKey('auth.User')
    interval = models.CharField(max_length=20, choices=NOTIFICATION_INTERVALS)

    def get_preferred_email(self):
        return self.user.email


class WrongTestProfile(models.Model):
    """Wrong enhanced ``User`` model to test profile import."""
    pass


class EmptyAggregation(object):
    """Empty class to aggregate users."""
    pass



"""Template tags for the ``object_events`` app."""
from django import template

from ..models import ObjectEvent

register = template.Library()


@register.inclusion_tag('object_events/notifications.html', takes_context=True)
def render_notifications(context, notification_amount=8):
    """Template tag to render fresh notifications for the current user."""
    if context.get('request') and context['request'].user.is_authenticated():
        events = ObjectEvent.objects.filter(user=context['request'].user)
        if events:
            return {
                'authenticated': True,
                'request': context['request'],
                'unread_amount': events.filter(read_by_user=False).count(),
                'notifications': events[:notification_amount],
            }
        return {'authenticated': True}
    return {}



"""Factories of the ``object_events`` app."""
import factory

from django.utils.timezone import now

from django_libs.tests.factories import UserFactory

from ..models import ObjectEvent, ObjectEventType
from .test_app.models import DummyModel, TestProfile


class DummyModelFactory(factory.DjangoModelFactory):
    """Factory for the ``DummyModel`` model."""
    FACTORY_FOR = DummyModel

    name = 'Foobar'


class TestProfileFactory(factory.DjangoModelFactory):
    """Factory for the ``TestProfile`` model."""
    FACTORY_FOR = TestProfile

    user = factory.SubFactory(UserFactory)


class ObjectEventTypeFactory(factory.DjangoModelFactory):
    FACTORY_FOR = ObjectEventType

    title = factory.Sequence(lambda x: 'event_type{0}'.format(x))


class ObjectEventFactory(factory.DjangoModelFactory):
    FACTORY_FOR = ObjectEvent

    user = factory.SubFactory(UserFactory)
    creation_date = factory.LazyAttribute(lambda x: now())
    event_type = factory.SubFactory(ObjectEventTypeFactory)
    content_object = factory.SubFactory(DummyModelFactory)



"""Tests for views of the ``object_events``` application."""
from django.core.urlresolvers import reverse
from django.test import TestCase

from django_libs.tests.factories import UserFactory
from django_libs.tests.mixins import ViewTestMixin

from ..factories import ObjectEventFactory
from ...models import ObjectEvent


class ObjectEventsListViewTestCase(ViewTestMixin, TestCase):
    """Tests for the ``ObjectEventsListView`` view."""
    longMessage = True

    def setUp(self):
        self.user = UserFactory()

    def get_view_name(self):
        return 'object_events_list'

    def test_view(self):
        self.should_be_callable_when_authenticated(self.user)


class ObjectEventsMarkViewTestCase(ViewTestMixin, TestCase):
    """Tests for the ``ObjectEventsMarkView`` view."""
    longMessage = True

    def setUp(self):
        self.user = UserFactory()
        self.event = ObjectEventFactory(user=self.user)

    def get_view_name(self):
        return 'object_events_mark'

    def test_view(self):
        self.is_not_callable(user=self.user)
        self.is_callable(method='post',
                         and_redirects_to=reverse('object_events_list'))

        # ID is no integer
        self.is_not_callable(method='post', data={'single_mark': 'abc'})

        # ID is non-existant
        self.is_not_callable(method='post', data={'single_mark': 999})

        # Successful post and redirect to custom url
        self.is_callable(method='post', and_redirects_to='/test/',
                         data={'single_mark': self.event.pk, 'next': '/test/'})
        self.assertTrue(ObjectEvent.objects.get(pk=self.event.pk).read_by_user)

        # Succesful single-mark AJAX post
        resp = self.client.post(self.get_url(), {'single_mark': self.event.pk},
                                HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(resp.content, 'marked')

        # bulk_mark variable has no pk items
        self.is_callable(method='post', data={'bulk_mark': 'abc, x, []'},
                         and_redirects_to=reverse('object_events_list'))

        # bulk_mark variable is an empty string
        self.is_callable(method='post', data={'bulk_mark': ''},
                         and_redirects_to=reverse('object_events_list'))

        # Successful bulk_mark post
        e2 = ObjectEventFactory(user=self.user)
        e3 = ObjectEventFactory(user=self.user)
        self.is_callable(method='post',
                         data={'bulk_mark': '{0}, {1}, '.format(e2.pk, e3.pk)},
                         and_redirects_to=reverse('object_events_list'))
        self.assertTrue(ObjectEvent.objects.get(pk=e2.pk).read_by_user)
        self.assertTrue(ObjectEvent.objects.get(pk=e3.pk).read_by_user)

        # Succesful bulk-mark AJAX post
        resp = self.client.post(self.get_url(), {'bulk_mark': e2.pk},
                                HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(resp.content, 'marked')
